<#  
.SYNOPSIS  
	Shows information related to current voice calls taking place.

.DESCRIPTION  
	Shows information related to current voice calls taking place. It requires the LCSCDR database, which is present when the Monitoring Server role is installed in the topology.

.NOTES  
  Version      				: 1.2
  Rights Required			: Local admin on server
  										: ExecutionPolicy of RemoteSigned or Unrestricted
  Lync Version				: 2010 (tested on CU 4 and 5)
  Author       				: Pat Richard, Exchange MVP
  Email/Blog/Twitter	: pat@innervation.com 	http://www.ehloworld.com @patrichard
  Dedicated Blog			: http://www.ehloworld.com/1179
  Disclaimer   				: You running this script means you won't blame me if this breaks your stuff.
  Info Stolen from 		: 

.EXAMPLE
	Get-CsVoiceCalls -server [SQL server name]
	
	Details
	=======
	Connectes to the SQL server hosting the LCSCDR database, and gathers information for display
	
.INPUTS
	None. You cannot pipe objects to this script.

#Requires -Version 2.0
#>

	[CmdletBinding(SupportsShouldProcess = $True)]
	Param (
		[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True, Mandatory = $False)]
		[ValidateNotNullOrEmpty()]
		[string]$server,
		[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True, Mandatory = $False)]
		[ValidateNotNullOrEmpty()] 
		[string]$database = "master",
		[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True, Mandatory = $False)]
		[ValidateNotNullOrEmpty()] 
		[string]$query2 = "select s.[SessionIdTime],u1.UserUri as User1Uri,[User1Id],u2.UserUri as User2Uri,[User2Id],[User1EndpointId],[User2EndpointId],[TargetUserId],[SessionStartedById],[OnBehalfOfId],[ReferredById],[IsUser1Internal],[IsUser2Internal],[ResponseTime],[DiagnosticId] FROM [LcsCDR].[dbo].[SessionDetails] s left outer join [LcsCDR].[dbo].[Users] u1 on s.User1Id = u1.UserId left outer join [LcsCDR].[dbo].[Users] u2 on s.User2Id = u2.UserId where ResponseCode=200 and s.SessionEndTime is null and (User1Id is null or User2Id is null or User1Id != User2Id) and MediaTypes = 16 and s.[SessionIdTime] >= dateadd(dd,0, datediff(dd,0, getDate()))", 
		[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True, Mandatory = $False)]
		[ValidateNotNullOrEmpty()] 
		[string]$query = "select s.[SessionIdTime],u1.UserUri as User1Uri,[User1Id],u2.UserUri as User2Uri,[User2Id],[TargetUserId],[SessionStartedById],[OnBehalfOfId],[ReferredById],[IsUser1Internal],[IsUser2Internal],[ResponseTime],[DiagnosticId] FROM [LcsCDR].[dbo].[SessionDetails] s left outer join [LcsCDR].[dbo].[Users] u1 on s.User1Id = u1.UserId left outer join [LcsCDR].[dbo].[Users] u2 on s.User2Id = u2.UserId where ResponseCode=200 and s.SessionEndTime is null and (User1Id is null or User2Id is null or User1Id != User2Id) and MediaTypes = 16 and s.[SessionIdTime] >= dateadd(dd,0, datediff(dd,0, getDate()))", 
		[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True, Mandatory = $False)]
		[ValidateNotNullOrEmpty()]
		[string]$ConnectionName = "Lync SQL Query", 
		[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True, Mandatory = $False)]
		[ValidateNotNullOrEmpty()]
		[int]$CommandTimeout = 15,
		[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $False, Mandatory = $False)]
		[ValidateNotNullOrEmpty()]
		[ValidateRange(-12,14)]		
		[int]$GMTOffsetHours = ([System.TimeZoneInfo]::Local).BaseUtcOffset.Hours,
		[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $False, Mandatory = $False)]
		[ValidateNotNullOrEmpty()]
		[ValidateRange(0,60)]
		[int]$GMTOffsetMinutes = ([System.TimeZoneInfo]::Local).BaseUtcOffset.Minutes
	)
	
	if ((Get-Date).IsDaylightSavingTime()){
		$GMTOffsetHours = $GMTOffsetHours+1
	}
	
	Write-Verbose "Validating that database exists"	
	if ((Get-WMIObject -Class "Win32_PerfFormattedData_MsSqlServer_SqlServerDatabases" Computer $server | ? {$_.name -match "lcscdr"}) -ne $null){
		Write-Verbose "Database exists on $server"	
		$conn = New-Object ('System.Data.SqlClient.SqlConnection') 
		[string]$ConnString = "Server=$server;Integrated Security=SSPI;Database=$database;Application Name=$connectionName"
		$conn.ConnectionString = $ConnString
		$x = 0
		Write-Debug ("Function: Get-CsVoiceCalls: $server $database") 
		if (Test-Path variable:\conn) { 
			$conn.close() 
		} else { 
			$conn = New-Object ('System.Data.SqlClient.SqlConnection') 
		} 
		$conn.Open() 
		Write-Verbose "Creating connection"
		$sqlCmd = New-Object System.Data.SqlClient.SqlCommand 
		$sqlCmd.CommandTimeout = $CommandTimeout
		$sqlCmd.CommandText = $query
		$sqlCmd.Connection = $conn
		Write-Verbose "Executing query"
		$data = $sqlCmd.ExecuteReader()
		while ($data.read() -eq $true) { 
			$max = $data.FieldCount -1 
			$obj = New-Object Object 
			For ($i = 0; $i -le $max; $i++) { 
				$name = $data.GetName($i) 
				$value = $data.GetValue($i)
				if ($name.length -eq 0) { 
					$name = "field$i"
				} 
				if ($name -eq "SessionIdTime"){
					$value = (Get-Date $value).AddHours($GMTOffsetHours)
					$value = (Get-Date $value).AddMinutes($GMTOffsetMinutes)
				}
				if ($name -eq "ResponseTime"){					
					$value = (Get-Date $value).AddHours($GMTOffsetHours)
					$value = (Get-Date $value).AddMinutes($GMTOffsetMinutes)	
				}
				if (($name -eq "user2uri") -and (!($value.length -gt 1))){
					$obj | Add-Member Noteproperty $name -value "PSTN call" -force
				}elseif (($name -eq "user1uri") -and (!($value.length -gt 1))){
					$obj | Add-Member Noteproperty $name -value "PSTN caller" -force
				}else{			
					$obj | Add-Member Noteproperty $name -value $value -force
				}
			} 
			$obj
			$x++
		} 
		Write-Host "Total current voice calls: $x"
		$conn.close() 
		$conn = $null
	}else{
		Write-Host "Database does not exist"
	}

